﻿# -----------------------------------------------------------------------------------------------------------------------

# Etude de la charge du circuit RC

# Acquisition de la tension aux bornes du condensateur (broche A0) puis affichage des données

# Penser à téléverser le programme "StandardFirmata" sur la carte Arduino avant de lancer le programme Python

# (sur l'IDE Arduino aller dans : Fichier/Exemples/Firmata/StandardFirmata)

# -----------------------------------------------------------------------------------------------------------------------





# Importation des bibliothèques

from pyfirmata import Arduino, util     # Communication avec Arduino via le port série

from pylab import *                     # Fonctions mathématiques, tracé des courbes

import time                             # Mesure des durées





# Initialisation des variables

t_max = 12              # Durée d'acquisition (s)

N = 500                 # Nombre de points enregistrés

flag_Tau = False        # "Drapeau" = faux tant que l'on n'a pas atteint 63.2% de 5.0 V (détermination de Tau)

temps = 0               # t0 = 0

dt = t_max/N            # Incrément de temps

t = []                  # liste vide pour stocker les valeurs de t

Uc = []                 # liste vide pour stocker les valeurs de Uc





# Connexion au port série et début de l'acquisition

print("Connexion à la carte Arduino en cours...")

CarteArduino = Arduino('COM5')                      # Carte Arduino connectée au port "COM5"

Acquisition = util.Iterator(CarteArduino)

Acquisition.start()                                 # Début de l'acquisition



# Déclaration de la broche A0 en tant qu'entrée

BrocheA0 = CarteArduino.get_pin('a:0:i')    # "a:0" pour A0 et "i" pour "input"

time.sleep(0.5)                             # Attente de 0.5 s



print("La connexion est établie !")

print("La carte est prête à procéder à l'acquisition pendant ", t_max," s")

print("L\'acquisition commencera automatiquement dès le début de la charge du condensateur")



# Attente que le seuil de 5 mV soit atteint pour demarrer l'enregistrement des données

while (BrocheA0.read()*5 < 0.005) :

    time.sleep(0.01)



t0 = time.time()  # t0 = date de démarrage de l'enregistrement





# Enregistrement des données (t,Uc) jusqu'à t = tmax -------------------------------------------------------------------



while (temps <= t_max):



    # Détermination puis ajout de la variable "temps" à la liste "t"

    temps = time.time() - t0

    t.append(temps)





    # Lecture de "Tension" sur la broche A0 puis ajout à la liste "Uc"

    # Attention : "BrocheA0.read()" retourne une valeur entre 0 et 1 !

    Tension = BrocheA0.read()*5

    Uc.append(Tension)



    # Détermination de "Tau"

    if (flag_Tau == False and Tension >= 0.632*5) :  # Si "Tau" n'a pas encore été déterminé et que Uc >= 63.2% de 5.0 V

        Tau = temps                                  # Alors on vient d'atteindre t = Tau

        flag_Tau = True                              # On déclare que "Tau" a été déterminé

    print(round(Tension,2))



    # Attente pendant "dt" secondes

    time.sleep(dt)



# Fin de l'enregistrement des données -----------------------------------------------------------------------------------





CarteArduino.exit()



# Création de la liste des valeurs de Uc théoriques

Uc_modele = [5.0*(1-exp(-temps/Tau)) for temps in t]





# Affichage des données ------------------------------------------------------------------------------------------------



# Affichage du temps caractéristique du circuit RC

print("Constante de temps (63,2%) : Tau = ",round(Tau,2)," s")



# Affichage graphique des données

title('Etude du temps caractéristique d\'un circuit RC')

plot(t,Uc,'b+', label="Uc mesurée")

plot(t,Uc_modele,'r', label="Uc modélisée")

legend()

xlabel('temps (s)')

ylabel('Uc (V)')

axis([0,t_max,0,5])

show()

# -----------------------------------------------------------------------------------------------------------------------